// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// All Rights Reserved. 
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

// prototying the XMLDocument  
XMLDocument.prototype.selectNodes = function(cXPathString, xNode)
{
	if( !xNode ) { xNode = this; } 

	var oNSResolver = this.createNSResolver(this.documentElement)
	var aItems = this.evaluate(cXPathString, xNode, oNSResolver, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null)
	var aResult = [];
	for( var i = 0; i < aItems.snapshotLength; i++)
	{
		aResult[i] =  aItems.snapshotItem(i);
	}
	
	return aResult;
}

XMLDocument.prototype.selectSingleNode = function(cXPathString, xNode)
{
	if( !xNode ) { xNode = this; } 

	var xItems = this.selectNodes(cXPathString, xNode);
	if( xItems.length > 0 )
	{
		return xItems[0];
	}
	else
	{
		return null;
	}
}

Element.prototype.selectNodes = function(cXPathString)
{
	if(this.ownerDocument.selectNodes)
	{
		return this.ownerDocument.selectNodes(cXPathString, this);
	}
	else{throw "For XML Elements Only";}
}

Element.prototype.selectSingleNode = function(cXPathString)
{	
	if(this.ownerDocument.selectSingleNode)
	{
		return this.ownerDocument.selectSingleNode(cXPathString, this);
	}
	else{throw "For XML Elements Only";}
}

function thumbstripsXmlHelper()
{
    this.m_xmlData = null;
}

thumbstripsXmlHelper.prototype.Initialize  = thumbstripsXmlHelperInitialize;
thumbstripsXmlHelper.prototype.Load = thumbstripsXmlHelperLoad;
thumbstripsXmlHelper.prototype.LoadFromFile = thumbstripsXmlHelperLoadFromFile;
thumbstripsXmlHelper.prototype.Save = thumbstripsXmlHelperSave;
thumbstripsXmlHelper.prototype.SaveToFile = thumbstripsXmlHelperSaveToFile;
thumbstripsXmlHelper.prototype.AddTextNode = thumbstripsXmlHelperAddTextNode;
thumbstripsXmlHelper.prototype.AddChildNode = thumbstripsXmlHelperAddChildNode;
thumbstripsXmlHelper.prototype.Serialize = thumbstripsXmlHelperSerialize;

function thumbstripsXmlHelperInitialize(xmlNamespace, rootNode)
{
    if(this.m_xmlData==null)
    {
        // code for IE
        if (window.ActiveXObject)
        {
            this.m_xmlData=new ActiveXObject("Microsoft.XMLDOM");
            this.m_xmlData.async=false;
        }
        else if (document.implementation && document.implementation.createDocument)
        {
            this.m_xmlData=document.implementation.createDocument(xmlNamespace,rootNode,null);
        }
    }
}    
    
function thumbstripsXmlHelperLoad(parentDirectory, fileName)
{
    var thumbstripsPath = "";
    
    if(this.m_xmlData==null)
    {
        this.Initialize("", "");
    }
    
    if(true) // placeholder for ie/firefox split
    {
        var targetFile = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("UChrm", Components.interfaces.nsIFile);
        
        // History file stored at <user's chrome>\filmstrips\<file_name><file_ext>
        targetFile.append(parentDirectory);

        if(!targetFile.exists()) 
        {
	        targetFile.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0664);	
        }

        targetFile.append(fileName);
    
    	var ios = Components.classes["@mozilla.org/network/io-service;1"]
				.getService(Components.interfaces.nsIIOService);
	    var fileHandler = ios.getProtocolHandler("file")
				.QueryInterface(Components.interfaces.nsIFileProtocolHandler);
				
	    thumbstripsPath = fileHandler.getURLSpecFromFile(targetFile);
    }
    
    this.m_xmlData.async = false;
    this.m_xmlData.load(thumbstripsPath);
}

function thumbstripsXmlHelperLoadFromFile(targetFile)
{
    var thumbstripsPath = "";
    
    if(this.m_xmlData==null)
    {
        this.Initialize("", "");
    }
    
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
				.getService(Components.interfaces.nsIIOService);
	var fileHandler = ios.getProtocolHandler("file")
				.QueryInterface(Components.interfaces.nsIFileProtocolHandler);
				
	thumbstripsPath = fileHandler.getURLSpecFromFile(targetFile);
    
    this.m_xmlData.async = false;
    this.m_xmlData.load(thumbstripsPath);
}

function thumbstripsXmlHelperSave(parentDirectory, fileName)
{
    if(this.m_xmlData!=null)
    {     
        if(true) // placeholder for a test for firefox or IE browser
        {                                
        	var targetFile = Components.classes["@mozilla.org/file/directory_service;1"]
                       .getService(Components.interfaces.nsIProperties)
                       .get("UChrm", Components.interfaces.nsIFile);
        
            // History file stored at <user's chrome>\filmstrips\<file_name><file_ext>
            targetFile.append(parentDirectory);
  
            if(!targetFile.exists()) 
            {
    	        targetFile.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0664);	
            }
            
            targetFile.append(fileName);
        	
        	var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
					.createInstance(Components.interfaces.nsIFileOutputStream);
		
            // use 0x02 | 0x10 to open file for appending.
	        foStream.init(targetFile, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
        	
	        // new: convert file to utf-8
	        var us = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
	                           .createInstance(Components.interfaces.nsIConverterOutputStream);
        	
	        us.init(foStream, "utf-8", 0, 0xFFFD); // U+FFFD = replacement character
	        us.writeString(this.Serialize(this.m_xmlData.documentElement));
	        us.close(); 
        }
    }
}

function thumbstripsXmlHelperSaveToFile(targetFile)
{
    if(this.m_xmlData!=null)
    {     
        if(true) // placeholder for a test for firefox or IE browser
        {                                        	
        	var foStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
					.createInstance(Components.interfaces.nsIFileOutputStream);
		
            // use 0x02 | 0x10 to open file for appending.
	        foStream.init(targetFile, 0x02 | 0x08 | 0x20, 0664, 0); // write, create, truncate
        	
	        // new: convert file to utf-8
	        var us = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
	                           .createInstance(Components.interfaces.nsIConverterOutputStream);
        	
	        us.init(foStream, "utf-8", 0, 0xFFFD); // U+FFFD = replacement character
	        us.writeString(this.Serialize(this.m_xmlData.documentElement));
	        us.close(); 
        }
    }
}

function thumbstripsXmlHelperAddTextNode(parentNodeName, childNodeTag, childNodeText)
{
    if(this.m_xmlData!=null)
    {
        foundElements = this.m_xmlData.getElementsByTagName(parentNodeName);
        
        if(foundElements!=null)
        {
            if(foundElements.length > 0)
            {
                childNode = this.m_xmlData.createElement(childNodeTag);
                textNode = this.m_xmlData.createTextNode(childNodeText);
                childNode.appendChild(textNode);
                foundElements[0].appendChild(childNode);
            }
        }
    }
}

function thumbstripsXmlHelperAddChildNode(parentNodeName, childNode)
{
    if(this.m_xmlData!=null)
    {
        foundElements = this.m_xmlData.getElementsByTagName(parentNodeName);
        
        if(foundElements!=null)
        {
            if(foundElements.length > 0)
            {
                foundElements[0].appendChild(childNode);
            }
         }
    }
}

function thumbstripsXmlHelperSerialize(startNode)
{
    var retVal = "";
    
    if (typeof XMLSerializer != "undefined")
        retVal = (new XMLSerializer( )).serializeToString(startNode);
    else if (startNode.xml) 
        retVal = startNode.xml;
        
    return retVal;
}	

function thumbstripsXmlHelperDeserialize(serializedXml)
{
    var retVal = new thumbstripsXmlHelper();
    
    // code for IE
    if (window.ActiveXObject)
    { 
        retVal.Initialize("","");        
        retVal.m_xmlData.async="false";
        retVal.m_xmlData.loadXML(serializedXml);
    }
    else
    {
        var parser = new DOMParser();
        retVal.m_xmlData = parser.parseFromString(serializedXml,"text/xml");
        parser = null;
    }
    
    return retVal;
}	

