// ********************************************************************************** //
// The contents of this file are subject to the Mozilla Public License Version 1.1    //
// (the "License"); you may not use this file except in compliance with the License.  //
// You may obtain a copy of the License at http://www.mozilla.org/MPL/                //
//                                                                                    //
// Software distributed under the License is distributed on an "AS IS" basis,         //
// WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License for the  //
// specific language governing rights and limitations under the License.              //
//                                                                                    //
// The Original Code provided by Mozilla's Firefox is XUL, Javascript, and CSS.       // 
// The Initial Developer of the Original Code is Aseem Kishore.                       //
// Portions created by Scott Williamson are Copyright (c) 2006.  All Rights Reserved. //
// Contributor - Intuit Inc. (c) 2007                                                 //
// ********************************************************************************** //

function GetCurrentUrl()
{
    return content.location.href.toString();
}

// Logging messages to the Javascript console
var thumbstrips_console = Components.classes['@mozilla.org/consoleservice;1']
		.getService(Components.interfaces.nsIConsoleService);

function thumbstrips_log(message) {
	thumbstrips_console.logStringMessage("[thumbstrips] " + message);
}


// Working with strings

function thumbstrips_isBlankSpace(c) {
	return (c === ' ' || c === '\n' || c === '\t');
}

function thumbstrips_removeComments(text) 
{
	if (!text) 
	{
		return "";
	}
	
	var retVal = "";
	
	retVal = text;
	
	var startPos = text.search("<!--");		
	var endPos = text.search("-->");
	
	if(startPos < 0)
	    startPos = 0;
	else 
	    startPos = 4;
	
	if(endPos < 0)
	    endPos = text.length;
	else
	    endPos = text.length - 3;
	
	retVal = text.slice(startPos, endPos);

	var regexp = new RegExp("--", "g");	
	return retVal.replace(regexp, " - ");
}

function thumbstrips_removeDashes(text) {
	return text.replace(new RegExp("--", "g"), " - ");
}

function thumbstrips_trim(string) {
	if (!string) {
		return "";
	}
	
	var n = string.length;
	var trim = "";
	var c;
	
	for (i = 0; i < n; i++) {
		
		// first remove blank space at the beginning
		
		var x = string[i];
		
		if (!thumbstrips_isBlankSpace(x)) {
			trim = x;
			c = i;
			break;
		}
		
	}
	
	if (trim === "") { // this means the whole string was blank spaces
		return "";
	}
	
	var blankSpaceFlag = false;
	
	for (i = c + 1; i < n; i++) {
		
		// go through each character and copy it
		// if we see a blank space, set the flag to true and ignore every
		// OTHER blank space until we see a normal character again
		
		var x = string[i];
		
		if (blankSpaceFlag === true) {
			if (!thumbstrips_isBlankSpace(x)) {
				blankSpaceFlag = false;
				trim += x;
			}
		} else {
			// always copy, even if it's a blank space
			// i.e. we want to copy the FIRST blank space
			
			if (thumbstrips_isBlankSpace(x)) {
				blankSpaceFlag = true;
				trim += " "; // replace \n and \t with space
			} else {
				trim += x;
			}
		}
	}
	
	if (trim[trim.length - 1] === " ") {
		trim = trim.slice(0, trim.length - 1);
	}
	
	return trim;
}

// only remove leading and trailing spaces
function thumbstrips_simpleTrim(string) {
	if (!string) {
		return "";
	}
	
	var n = string.length;
	var c1 = -1; // the index of the first non-blank space
	var c2 = n; // the index of the last non-blank space
	
	for (i = 0; i < n; i++) {
		var x = string[i];
		
		if (!thumbstrips_isBlankSpace(x)) {
			c1 = i;
			break;
		}
	}
	
	if (c1 === -1) { // this means the whole string was blank spaces
		return "";
	}
	
	for (i = (n - 1); i >= 0; i--) {
		var x = string[i];
		
		if (!thumbstrips_isBlankSpace(x)) {
			c2 = i;
			break;
		}
	}
	
	// c2 should NOT be n anymore -- that's only the case if the whole string
	// is blank spaces, and we took care of that case above
	
	return string.slice(c1, c2 + 1);
}

// Working with arrays

function thumbstrips_copy(array) {
	var copy = new Array(array.length);
	
	for (var i in array) {
		copy[i] = array[i];
	}
	
	return copy;
}

function thumbstrips_propCopy(array) {
	var copy = new Array();
	
	for (var i in array) {
		copy[copy.length] = array[i];
	}
	
	return copy;
}

function thumbstrips_propLength(array) {
	var c = 0;

	for (var i in array) {
		c++;
	}
	
	return c;
}

function thumbstrips_clearArray(array) {
	thumbstrips_checkArrayType("clearArray", array);
	
	array.length = 0;
	
	for (var i in array) {
		delete array[i];
	}
}

// Validating input data
function thumbstrips_checkTrue(from, statement, description) {
	var e = "errors.js / checkTrue / " + from + " ::: ";
	
	if (statement !== true) {
		throw new Error(e + description);
	}
}

function thumbstrips_checkType(from, x, type) {
	var e = "errors.js / checkType / " + from + " ::: ";
	
	if ((typeof type) != "string") {
		throw new TypeError(e + "type is not a string / " +
				"type = " + type + " / " +
				"(typeof type) = " + (typeof type));
	}
	
	if ((typeof x) != type) {
		throw new TypeError(e + "x is not proper type / " +
				"x = " + x + " / " +
				"type = " + type + " / " +
				"(typeof x) = " + (typeof x));
	}
}

function thumbstrips_checkArrayType(from, x) {
	var e = "errors.js / checkArrayType / " + from + " ::: ";
	
	if (!(x instanceof Array)) {
		throw new TypeError(e + "x is not an array / " +
				"x = " + x + " / " +
				"(typeof x) = " + (typeof x));
	}
}

function thumbstrips_checkDateType(from, x) {
	var e = "errors.js / checkDateType / " + from + " ::: ";
	
	if (!(x instanceof Date)) {
		throw new TypeError(e + "x is not a Date / " +
				"x = " + x + " / " +
				"(typeof x) = " + (typeof x));
	}
}

// Comparison functions

// Time calculations

/**
 * Returns an array that breaks down the time duration (given as a number of
 * milliseconds) into seven slots representing the number of years, months,
 * days, hours, minutes, seconds and milliseconds, in that order, that have
 * passed within that duration.
 * 
 * Note: these slots act like mixed fractions, so as a result: the number of
 * milliseconds will never exceed 999; the number of seconds will never exceed
 * 59; the number of minutes will never exceed 59; the number of hours will
 * never exceed 23; the number of days will never exceed 30 (nor will it exceed
 * 29 on months with 30 days, 28 on February in leap years, or 27 on February in
 * a non-leap year); and the number of months will never exceed 11.
 * 
 * An exception to these rules is if a precision is given. The precision may be
 * "year" (default), "month", "day", "hour", "minute" or "second", which
 * prevents the slots to the left of the given precision from being filled.
 * 
 * For example, for a time duration of 86,461,450, a precision of "day" yields
 * an array representing 1 day, 0 hours, 1 minute, 1 second and 450 milliseconds
 * while a precision of "hour" yields 24 hours, 1 minute, 1 second and 450
 * milliseconds and a precision of "minute" yields 1441 minutes, 1 second and
 * 450 milliseconds.
 * 
 * NOTE: Currently, precision can only be "day", "hour", "minute" or "second",
 * and will default to "day" if no other or any other precision is given. The
 * portions for calculating the number of months and years have not been
 * implemented yet (due to tricky issues like leap years, uneven months, etc.).
 */
function thumbstrips_getTimeDurationAsArray(totalms, precision) {
	var array = new Array(7);
	var goodToGo = false;
	
	// some constants
	var millisecondsPerSecond = 1000;
	var secondsPerMinute = 60;
	var minutesPerHour = 60;
	var hoursPerDay = 24;
	
	// TODO number of years and months
	
	// for now, assume a precision of "day"
	switch (precision) {
		case "hour":
		case "minute":
		case "second":
			break;
			
		default:
			precision = "day";
	}
	
	// calculate the number of years
	if (goodToGo === true || precision === "year") {
		// ignore for now
		goodToGo = true;
	} else {
		array[0] = 0;
	}
	
	// calculate the number of months
	if (goodToGo === true || precision === "month") {
		// ignore for now
		goodToGo = true;
	} else {
		array[1] = 0;
	}
	
	// calculate the number of days
	if (goodToGo === true || precision === "day") {
		var millisecondsPerDay = millisecondsPerSecond * secondsPerMinute * minutesPerHour * hoursPerDay;
		
		// verify it's 86,400,000
		if (millisecondsPerDay !== 86400000) {
			alert("ERROR! ms/day = " + millisecondsPerDay);
		}
		
		array[2] = Math.floor(totalms / millisecondsPerDay);
		totalms %= millisecondsPerDay;
		goodToGo = true;
	} else {
		array[2] = 0;
	}
	
	// calculate the number of hours
	if (goodToGo === true || precision === "hour") {
		var millisecondsPerHour = millisecondsPerSecond * secondsPerMinute * minutesPerHour;
		
		// verify it's 3,600,000
		if (millisecondsPerHour !== 3600000) {
			alert("ERROR! ms/hr = " + millisecondsPerHour);
		}
		
		array[3] = Math.floor(totalms / millisecondsPerHour);
		totalms %= millisecondsPerHour;
		goodToGo = true;
	} else {
		array[3] = 0;
	}
	
	// calculate the number of minutes
	if (goodToGo === true || precision === "minute") {
		var millisecondsPerMinute = millisecondsPerSecond * secondsPerMinute;
		
		// verify it's 60,000
		if (millisecondsPerMinute !== 60000) {
			alert("ERROR! ms/min = " + millisecondsPerMinute);
		}
		
		array[4] = Math.floor(totalms / millisecondsPerMinute);
		totalms %= millisecondsPerMinute;
		goodToGo = true;
	} else {
		array[4] = 0;
	}
	
	// calculate the number of seconds -- this will always get calculated
	array[5] = Math.floor(totalms / millisecondsPerSecond);
	totalms %= millisecondsPerSecond;
	
	// remainder is the number of milliseconds
	array[6] = totalms;
	
	return array;
}

function thumbstrips_getTimeDifferenceAsString(date1, date2, leftPrecision, rightPrecision) {
	return thumbstrips_getTimeDurationAsString(Math.abs(date2 - date1), leftPrecision, rightPrecision);
}

function thumbstrips_getTimeDurationAsString(totalms, leftPrecision, rightPrecision) {
	var array = thumbstrips_getTimeDurationAsArray(totalms, leftPrecision);
	var text = new Array();
	var stringBundle = document.getElementById("thumbstrips_utilStringBundle");
	
	switch (rightPrecision) {
		case "year":
		case "month":
		case "day":
		case "hour":
			break;
			
		case "second":
			rightPrecision = "sec";
			break;
		case "millisecond":
			rightPrecision = "millisec";
			break;
		
		default:
			rightPrecision = "min";
	}
	
	for (var i = 0; i < array.length; i++) {
		var string;
		
		switch (i) {
			case 0:
				string = (array[i] > 1) ? stringBundle.getString("years") : stringBundle.getString("year");
				break;
			case 1:
				string = (array[i] > 1) ? stringBundle.getString("months") : stringBundle.getString("month");
				break;
			case 2:
				string = (array[i] > 1) ? stringBundle.getString("days") : stringBundle.getString("day");
				break;
			case 3:
				string = (array[i] > 1) ? stringBundle.getString("hours") : stringBundle.getString("hour");
				break;
			case 4:
				string = (array[i] > 1) ? stringBundle.getString("mins") : stringBundle.getString("min");
				break;
			case 5:
				string = (array[i] > 1) ? stringBundle.getString("secs") : stringBundle.getString("sec");
				break;
			case 6:
				string = (array[i] > 1) ? stringBundle.getString("millisecs") : stringBundle.getString("millisec");
				break;
			default:
				thumbstrips_log("Error, invalid time, i = " + i);
				string = "???";
		}
		
		if (array[i] > 0) {
			text[text.length] = array[i] + " " + string;
		}
		
		// various breaking conditions
		if (i === 0 && rightPrecision === "year") {
			break;
		} else if (i === 1 && rightPrecision === "month") {
			break;
		} else if (i === 2 && rightPrecision === "day") {
			break;
		} else if (i === 3 && rightPrecision === "hour") {
			break;
		} else if (i === 4 && rightPrecision === "min") {
			break;
		} else if (i === 5 && rightPrecision === "sec") {
			break;
		} // unnecessary to check for millisec precision
	}
	
	if (text.length === 0) {
		return "< 1 " + rightPrecision;
	}
	
	string = text[0];
	
	for (var i = 1; i < text.length; i++) {
		string += ", " + text[i];
	}
	
	return string;
}

/**
 * Returns a string representing the time difference between the current date
 * and the given date, to the given precision. If no precision is given, the
 * default precision is to the minute.
 * 
 * Example: If date is Sunday, July 4th, 2006, 4:13:07 PM EST, and the date at
 * the time of the call is Sunday, August 6th, 2006, 11:48:37 PM EST, the return
 * value will be one of the following, depending on the precision:
 * 
 * year: < 1 year ago
 * month: 1 month ago
 * day: 1 month, 2 days ago
 * hour: 1 month, 2 days, 7 hours ago
 * minute: 1 month, 2 days, 7 hours, 35 minutes ago
 * second: 1 month, 2 days, 7 hours, 35 minutes, 30 seconds ago
 * millisecond: 1 month, 2 days, 7 hours, 35 minutes, 30 seconds, 457 milliseconds ago
 * 
 * @param date A Date object.
 * @param precision A string that is one of either "millisecond", "second",
 * 		"minute", "hour", "day", "month" or "year" that determines the cutoff
 * 		for precision. If none is given, this defaults to "minute".
 */
function thumbstrips_getTimeAgo(date, leftPrecision, rightPrecision) {
	var stringBundle = document.getElementById("thumbstrips_utilStringBundle");
	var ago = stringBundle.getString("ago");
	
	return thumbstrips_getTimeDifferenceAsString(date, new Date(), leftPrecision, rightPrecision) + " " + ago;
}

function thumbstrips_getTimeLength(totalms, leftPrecision, rightPrecision) {
	return thumbstrips_getTimeDurationAsString(totalms, leftPrecision, rightPrecision);
}


function thumbstrips_getFileName(path) {
	//alert("path: " + path);
	
	var fullPath  = path;
	
	// Windows OS
	if(navigator.userAgent.indexOf('Win') != -1) {
		var lastSlash = path.lastIndexOf("\\");
	}
	else if(navigator.userAgent.indexOf('Mac') != -1) {
		var lastSlash = path.lastIndexOf("/");
	}
	
	var fileName  = path.substring(lastSlash+1,path.length);
	var directory = path.substring(0,lastSlash);
	
	//alert("fileName: " + fileName);
	return fileName;
}


function thumbstrips_createShareEmail(from, file) {
	var email_subject    = "Thumbstrips for Firefox";
	
	/*var email_body_line1 = " has sent a ThumbStrip for you to check out!"; 
	var email_body_line2 = ">> If you haven&#146;t already installed the ThumbStrips Firefox extension, please do that first by clicking http://innovation.intuit.com/blog/thumbstrips";
	var email_body_line3 = ">> If you already have ThumbStrips installed, view the ThumbStrip by clicking here: http://innovation.intuit.com/sharing-download.php?link="+escape(file);
	var email_body_line4 = "What is ThumbStrips?  It&#146;s a Firefox add-on / extension that lets you view your web history in the form of a filmstrip of screenshots.  Go to http://innovation.intuit.com/blog/thumbstrips for more information about ThumbStrips, or go to http://innovation.intuit.com/blog/browse-for-thumbstrips to view more ThumbStrips from other folks.";
	var email_body_line5 = "We want to hear from you &#150; if you have some feedback, or just want to check out other cool applications in the works, head over to http://innovation.intuit.com and let us know what you think.";
	var email_body_line6 = "Thanks!";
	var email_body_line7 = "ThumbStrips team";
	var email_body_line8 = "(just another cool innovation by the Innovation Lab @ Intuit)";*/
	
	//var custom_link = "http://innovation.intuit.com/sharing-download.php?link="+escape(file);
	var custom_link = "http://innovation.intuit.com/blog/browse-for-thumbstrips?view="+escape(file);
	
	
    // Send email using default program
	//window.location.href="mailto:?subject="+email_subject+"&body="+from+email_body_line1+"%0A%0a"+email_body_line2+"%0A%0a"+email_body_line3+"%0A%0a"+email_body_line4+"%0A%0a"+email_body_line5+"%0A%0a"+email_body_line6+"%0A%0a"+email_body_line7+"%0A%0A"+email_body_line8+"\n";
 	//window.location.href="mailto:?subject="+email_subject+"&body="+from+"%20ThumbStrip%20for%20you%20to%20check%20out%21%20%20%0A%0A%3E%3E%20If%20you%20haven%27t%20already%20installed%20the%20ThumbStrips%20Firefox%20extension%2C%20please%20do%20that%20first%20by%20visiting%20http%3A//innovation.intuit.com/blog/thumbstrips%20%0A%0A%3E%3E%20If%20you%20already%20have%20ThumbStrips%20installed%2C%20view%20the%20ThumbStrip%20by%20clicking%20here%3A%20"+custom_link+"%0A%0AWhat%20is%20ThumbStrips%3F%20%20It%27s%20a%20Firefox%20add-on%20/%20extension%20that%20lets%20you%20view%20your%20web%20history%20in%20the%20form%20of%20a%20filmstrip%20of%20screenshots.%20%20Go%20to%20http%3A//innovation.intuit.com/blog/thumbstrips%20for%20more%20information%20about%20ThumbStrips%2C%20and%20to%20learn%20how%20you%20can%20view%20more%20ThumbStrips%20from%20other%20folks.%20%20%0A%0AWe%20want%20to%20hear%20from%20you%20-%20if%20you%20have%20some%20feedback%2C%20or%20just%20want%20to%20check%20out%20other%20cool%20applications%20in%20the%20works%2C%20head%20over%20to%20http%3A//innovation.intuit.com%20and%20let%20us%20know%20what%20you%20think.%20%0A%0AThanks%21%20%0AThumbStrips%20team%0A%28just%20another%20cool%20innovation%20by%20the%20Innovation%20Lab%20@%20Intuit%29";
    window.location.href="mailto:?subject="+email_subject+"&body="+from+"%20Thumbstrips%20for%20Firefox&Body=Here%27s%20a%20ThumbStrip%20for%20you%20to%20check%20out%21%20%20%0A%0A%3E%3E%20If%20you%20haven%27t%20already%20installed%20the%20ThumbStrips%20Firefox%20extension%2C%20please%20do%20that%20first%20by%20visiting%20http%3A//innovation.intuit.com/blog/thumbstrips.%20%20%0A%0A%3E%3E%20If%20you%20already%20have%20ThumbStrips%20installed%2C%20view%20the%20ThumbStrip%20by%20clicking%20here%3A%20"+custom_link+"%0A%0AWhat%20is%20ThumbStrips%3F%20%20It%27s%20a%20Firefox%20add-on%20/%20that%20lets%20you%20view%20and%20share%20web%20searches%20in%20the%20form%20of%20a%20filmstrip%20of%20screenshots.%20%20Go%20to%20http%3A//innovation.intuit.com/blog/thumbstrips%20for%20more%20information%20about%20ThumbStrips%2C%20and%20to%20learn%20how%20you%20can%20view%20more%20ThumbStrips%20from%20other%20folks.%20%20If%20you%20don%27t%20currently%20use%20the%20Firefox%20browser%2C%20you%20can%20get%20it%20for%20free%20by%20visiting%20http%3A//www.mozilla.com/en-US/%20and%20clicking%20on%20%22Download%20Firefox.%22%0A%0AWe%20want%20to%20hear%20from%20you%20-%20if%20you%20have%20some%20feedback%2C%20or%20just%20want%20to%20check%20out%20other%20cool%20applications%20in%20the%20works%2C%20head%20over%20to%20http%3A//innovation.intuit.com%20and%20let%20us%20know%20what%20you%20think.%20%0A%0AThanks%21%20%0AThumbStrips%20team%0A%0A%28just%20another%20cool%20innovation%20by%20the%20Innovation%20Lab%20@%20Intuit%29%20%0A%0A";
}